<?php

namespace Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Util\EmailController;

class Dashboard{
    // /** @var \Services\Settings */
    // private $settingService;
    // /** @var \Services\Password $passwordService */
    // protected $passwordService;
    // protected $entityManager;
    // protected $connection;
    
    public function __construct($connection, $environment, $newCode, $crudService) {
        $this->connection      = $connection;
        $this->environment     = $environment;
        $this->newCode         = $newCode;
        $this->crudService     = $crudService;
    }
    
    public function getDashboard(Request\Filter $request, Response $response) {
        $filters      = $request->getFilterCriteria()->getConditions();
        $stmt = $this->connection->prepare("
			SELECT CDOPERADOR, NMOPERADOR from OPERADOR"
		);
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("painel", $result));
    }
    
    public function getParametrizacaoPadrao(Request\Filter $request, Response $response) {
        $filters      = $request->getFilterCriteria()->getConditions();

        $stmt = $this->connection->prepare(
			"SELECT * FROM PAINEL"
		);
        $stmt->execute();
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("painel", $result));
    }
    
    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function findADConfiguration(Request\Filter $request, Response $response) {
        try {
            $adConfig = $this->settingService->findADConfigurationByNrOrg();
            $response->addDataSet(new DataSet ('adConfig', $adConfig));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    /**
     * @param Request\DataSet $request
     * @param Response $response
     */
    public function saveADConfiguration(Request\DataSet $request, Response $response) {
        try {
            $adRows      = $request->getDataSet()->getRows();
            $adRow       = $adRows[0];
            $useAd       = $adRow['USE_AD'];
            $domain      = $adRow['DOMAIN'];
            $server      = $adRow['SERVER'];
            $admUser     = isset($adRow['ADM_USER']) ? $adRow['ADM_USER'] : null;
            $admPassword = isset($adRow['ADM_PASSWORD']) ? $adRow['ADM_PASSWORD'] : null;
            $adConfig    = $this->settingService->saveADConfiguration($domain, $server, $admUser, $admPassword, $useAd);
            $response->addDataSet(new DataSet ('adConfig', $adConfig));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
    
    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function getOrganizationParameters(Request\Filter $request, Response $response) {
        try {
            $generalParameters  = $this->settingService->getOrganizationParameters();
            $generalParameters  = empty($generalParameters) ? array() : $generalParameters[0];
            $passwordParameters = $this->passwordService->findPasswordParameters();
            $passwordParameters = empty($passwordParameters) ? array() : $passwordParameters[0];
            $parameters = array_merge($generalParameters, $passwordParameters);
            $response->addDataSet(new DataSet ('parameters', array($parameters)));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    /**
     * @param Request\DataSet $request
     * @param Response $response
     */
    public function saveOrganizationParameters(Request\DataSet $request, Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $useRestriction = $row['USE_RESTRICTION'];
            $idUtCtrlSen    = $row['IDUTCTRLSEN'];
            $nrDiasExpSen   = $row['NRDIASEXPSEN'];
            $nrDiasAviso    = $row['NRDIASAVISO'];
            $nrAltSenhRep   = $row['NRALTSENHREP'];
            $cdPictSenha    = $row['CDPICTSENHA'];
            $nrErroSenha    = $row['NRERROSENHA'];
            $this->settingService->saveOrganizationParameters($useRestriction, $idUtCtrlSen, $nrDiasExpSen,
                $nrDiasAviso, $nrAltSenhRep, $cdPictSenha, $nrErroSenha);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
    
    public function getOrganizationRestrictionByIp(Request\Filter $request, Response $response){
        try {
            $filters      = $request->getFilterCriteria()->getConditions();
            $restrictions = $this->settingService->getOrganizationRestrictionByIp($filters);
            $response->addDataSet(new DataSet ('organizationRestrictionByIp', $restrictions));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
    
    public function saveOrganizationRestrictionByIp(Request\DataSet $request, Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $ipv4 = isset($row['IPV4']) ? $row['IPV4'] : null;
            $ipv6 = isset($row['IPV6']) ? $row['IPV6'] : null;
            $ipRestrictedId = isset($row['ID']) ? $row['ID'] : null;
            $active = isset($row['ACTIVE']) ? $row['ACTIVE'] : null;
            $description = isset($row['DESCRIPTION']) ? $row['DESCRIPTION'] : null;
            $restrictionArrayId = $this->settingService->saveOrganizationRestrictionByIp($ipRestrictedId, $ipv4, $ipv6, $description, $active);
            $response->addDataSet(new DataSet ('organizationRestrictionByIp', $restrictionArrayId));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
    
    public function removeOrganizationRestrictionByIp(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $restrictionId = $row['ID'];
            $this->settingService->removeOrganizationRestrictionByIp($restrictionId);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
    
    public function getIndicadoresVisiveis(Request\DataSet $request, Response $response) {
        $codOperador = $this->environment->getCdOperador();
        $filters = array(
            "P_CDOPERADOR" => $codOperador
        );
        $stmt = $this->connection->prepare(
            "SELECT E.NRPAINEL ,I.NRINDICPAINEL, I.IDVISUINDIC FROM PERFACESSPARC  P, OPERADOR O ,PERFACLPAINEL E , INDICPAINEL I 
             WHERE O.CDOPERADOR = :P_CDOPERADOR
               AND P.NRPARCNEGOCIO = O.NRPARCNEGOCIO
               AND E.NRPERFILACESSO = P.NRPERFILACESSO
               AND I.NRPERFACLPAINEL = E.NRPERFACLPAINEL"
		);
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("dashboard", $result));
    }
    
    public function getIndicadoresNaoVisiveis(Request\DataSet $request, Response $response) {
        $filters = array(
            "P_CDFILIAL" => $this->environment->getCdFilial(),
            "P_CDOPERADOR" => $this->environment->getCdOperador(),
            "P_NRORG" => $this->environment->getNrorg()
        );
        $stmt = $this->connection->prepare(
            "SELECT * FROM TABLE (OPERACESSOINDIC_DSB(:P_CDFILIAL, :P_CDOPERADOR, :P_NRORG))"
		);
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("indicadores", $result));
    }
    
    public function getExecao(Request\DataSet $request, Response $response) {
        $codOperador = $this->environment->getCdOperador();
        $filters = array(
            "P_CDOPERADOR" => $codOperador
        );
        $stmt = $this->connection->prepare(
            "SELECT NRINDICPAINEL 
              FROM INDICPAINEXCOPER
             WHERE CDOPERADOR = :P_CDOPERADOR"
		);
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("dashboard", $result));
    }
    
    //BUSCA O PAINEL PADRÃO (se não tiver acesso a nenhum painel, retorna null)
    public function getPainelPadrao(Request\DataSet $request, Response $response) {
        $filters = array(
            "P_CDFILIAL" => $this->environment->getCdFilial(),
            "P_CDOPERADOR" => $this->environment->getCdOperador(),
            "P_NRORG" => $this->environment->getNrorg()
        );
        $stmt = $this->connection->prepare(
           "SELECT NVL(E.NRPAINEL,MIN((SELECT MIN(T.NRPAINEL) 
                            FROM PERFACLPAINEL T 
                        WHERE T.NRPERFILACESSO = P.NRPERFILACESSO 
                        AND T.NRORG = :NRORG 
                        AND T.CDFILIAL = DECODE(PF.IDUTLDSBTPADRAO, 'N', :P_CDFILIAL, 'S', 'P', :P_CDFILIAL) ))) NRPAINEL, O.NRORG 
                    FROM OPERADOR O INNER JOIN PERFACESSPARC P ON P.NRPARCNEGOCIO = O.NRPARCNEGOCIO 
                    LEFT JOIN PERFACLPAINEL E ON E.NRPERFILACESSO = P.NRPERFILACESSO AND E.IDPADRAODASH = 'S'
                    INNER JOIN PARAMFILIAL PF ON PF.CDFILIAL = :P_CDFILIAL
                WHERE O.CDOPERADOR = :P_CDOPERADOR 
                    AND O.NRORG = :NRORG GROUP BY E.NRPAINEL, O.NRORG"
		);
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		
		$response->addDataSet(new DataSet("painelPadrao", $result));
    }
    
    public function setDataSourceGO(Request $request, Response $response) {
        $codOperador = $this->environment->getCdOperador();
        $filters = array(
            "P_DATAINI" => $currentTime = $this->environment->getCurrentTime()->format("01/m/Y"),
            "P_DATAFIN" => $currentTime = $this->environment->getCurrentTime()->format("t/m/Y"),
            "P_CDOPERADOR" => $codOperador
        );
        $stmt = $this->connection->prepare(
           "SELECT C.CDSUBGRPROD CDSUBGRPROD, C.NMSUBGRPROD NMSUBGRPROD, C.QTESTOQUE DIASEST, C.CDFILIAL, C.NMFILIAL
              FROM TABLE(CONSUMOD_ROP('T', TO_DATE(:P_DATAINI,'DD/MM/YYYY'), TO_DATE(:P_DATAFIN,'DD/MM/YYYY'), 'S', :P_CDOPERADOR)) C
             WHERE C.CDSUBGRPROD = 'TTTTTTTTTT'
               AND ROWNUM <= 10
             ORDER BY C.CDFILIAL, C.CDSUBGRPROD"
		);
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		
		$response->addDataSet(new DataSet("setDataSourceGO", $result));
    }
    
    public function setTabelaGO(Request $request, Response $response) {
        $codOperador = $this->environment->getCdOperador();
        $filters = array(
            "P_DATAINI" => $currentTime = $this->environment->getCurrentTime()->format("01/m/Y"),
            "P_DATAFIN" => $currentTime = $this->environment->getCurrentTime()->format("t/m/Y"),
            "P_CDOPERADOR" => $codOperador
        );
        $stmt = $this->connection->prepare(
            "SELECT C.CDSUBGRPROD CDSUBGRPROD, C.QTESTOQUE DIASEST, CASE WHEN C.CDSUBGRPROD = 'TTTTTTTTTT'
                                                                         THEN C.CDFILIAL
                                                                         ELSE NULL
                                                                    END AUXPARENT, 
                                                                    CASE WHEN C.CDSUBGRPROD = 'TTTTTTTTTT'
                                                                         THEN C.NMSUBGRPROD
                                                                         ELSE C.NMFILIAL
                                                                    END NOME, 
                                                                    CASE WHEN C.CDSUBGRPROD = 'TTTTTTTTTT'
                                                                         THEN NULL
                                                                         ELSE C.CDFILIAL
                                                                    END CDFILIAL
               FROM TABLE(CONSUMOD_ROP('T', TO_DATE(:P_DATAINI,'DD/MM/YYYY'), TO_DATE(:P_DATAFIN,'DD/MM/YYYY'), 'S', :P_CDOPERADOR)) C
              ORDER BY C.CDFILIAL, C.CDSUBGRPROD"
		);
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		
		$response->addDataSet(new DataSet("setTabelaGO", $result));
    }
    
    public function getAnalise(Request $request, Response $response) {
        $codOperador = $this->environment->getCdOperador();
        $filters = array(
             "P_CDOPERADOR" => $codOperador
            );
        $stmt = $this->connection->prepare(
            "SELECT FILIAL, DTPROCESSA, SERVICO, FILISERV, VRPUOBJETIVO, VRPUPLANEJADO, VRPUTENDENCIA, QTCOMESTMES, QTCOMESTPOS, QTEFETIVOS, VRDIFOBJPLA, VRDIFOBJTEND, VRPUPLAANT, VRPUREALANT, VRPUPLAPRXMES FROM (SELECT DISTINCT DM.CDFILIAL ||' - '||F.NMFILIAL AS FILIAL , DM.DTPROCESSA, DM.CDSERVFILI||' - '||SE.NMSERVICO AS SERVICO, DM.CDFILIAL ||' - '||F.NMFILIAL ||' '|| DM.CDSERVFILI||' - '||SE.NMSERVICO AS FILISERV, DM.VRPUOBJETIVO, DM.VRPUPLANEJADO, DM.VRPUTENDENCIA, DM.QTCOMESTMES, DM.QTCOMESTPOS, DM.QTEFETIVOS, DM.VRDIFOBJPLA, DM.VRDIFOBJTEND, DM.VRPUPLAANT, DM.VRPUREALANT, DM.VRPUPLAPRXMES FROM DM_PLANEJADORA DM INNER JOIN FILIAL F ON (DM.CDFILIAL = F.CDFILIAL) INNER JOIN SERVICO SE ON ((DM.CDFILIAL = SE.CDFILIAL) AND (DM.CDSERVFILI = SE.CDSERVFILI)) INNER JOIN FILIOPER FO ON FO.CDOPERADOR = :P_CDOPERADOR AND FO.CDFILIAL = F.CDFILIAL WHERE ((F.CDPLANEJADORA = :P_CDOPERADOR) OR (SE.IDSERVPRINC = 'S')) ORDER BY DM.VRPUPLANEJADO DESC, DM.VRPUTENDENCIA DESC, DM.VRPUOBJETIVO DESC, SERVICO) WHERE ROWNUM <= 20 ORDER BY SERVICO"
        );
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("getAnalise", $result));
    }
    
    public function getTabelaDesvio(Request $request, Response $response) {
        $filters = array(
             "P_CDFILIAL" => $this->environment->getCdFilial()
            );
        $stmt = $this->connection->prepare(
            "SELECT E.NMSERVICO, ROUND(E.VRCUSPLATOTSER1 - E.VRCUSREATOTSER1,2) VRDESVIO
                FROM ETL_RESUSERV E, (SELECT CASE WHEN P.IDUTILDTPROC = 'S' THEN NVL(PF.DTPROCESSA, SYSDATE) ELSE SYSDATE END DTPROCESSA
                FROM PARAMFILIAL PF, PARAMGERAL P
                WHERE PF.CDFILIAL = :P_CDFILIAL) F
                WHERE E.CDFILIAL = :P_CDFILIAL
                AND TO_DATE('01'||'/'||CDMESBI||'/'||CDANOBI, 'DD/MM/YYYY') = FIRST_DAY(F.DTPROCESSA)
                ORDER BY E.NMSERVICO"
        );
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("getTabelaDesvio", $result));
    }
    
    public function getTabelaOrcaConsumo(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $dataDeProcessamento = $row['DTPROCESSA'];
        $idtpdemanda = $row['IDTPDEMANDA'];
        
        $filters = array(
            "P_DATA" => $dataDeProcessamento,
            "P_CDFILIAL" => $this->environment->getCdFilial(),
            "P_IDORIGEM" => "CONSUMO",
            "P_IDTPDEMANDA" => $idtpdemanda,
            "NRORG" => $this->environment->getNrorg()
        );
        $stmt = $this->connection->prepare(
            "SELECT O.IDORIGEM, O.IDTPDEMANDA, O.CDNVLARVPROD, SUM(O.VRORCAREALOC) VRORCAREALOC, SUM(O.VRREALIZADO) VRREALIZADO, SUM(O.VRORCAMENTO) VRORCAMENTO
                  FROM ORCAMENTREALOC O INNER JOIN PARAMFILIAL PF ON O.CDFILIAL = PF.CDFILIAL
                                                                 AND O.NRORG = PF.NRORG
                                        INNER JOIN PARAMGERAL P ON P.NRORG = O.NRORG
                                        INNER JOIN PARAMORCAOPERAC OP ON OP.NRORG = O.NRORG
                                        INNER JOIN FILIAL F ON F.CDFILIAL = O.CDFILIAL
                 WHERE P.CDPARGERAL = '0001'
                   AND O.IDORIGEM = :P_IDORIGEM --'CONSUMO'
                   AND O.CDFILIAL = :P_CDFILIAL
                   AND O.IDTPDEMANDA = :P_IDTPDEMANDA
                   AND O.NRORG = :NRORG
                   AND LTRIM(TO_CHAR(O.DTMESPERIODO,'MM/YYYY')) = LTRIM(TO_CHAR(:P_DATA,'MM/YYYY'))
                   AND NOT EXISTS (SELECT 1
                                     FROM ORCAMENTREALOC M
                                    WHERE M.NRORCAMENTREALOC <> O.NRORCAMENTREALOC
                                      AND M.CDFILIAL = O.CDFILIAL
                                      AND M.IDTPDEMANDA = O.IDTPDEMANDA
                                      AND M.IDORIGEM = O.IDORIGEM
                                      AND M.DTMESPERIODO = O.DTMESPERIODO
                                      AND ((M.CDNVLARVPROD LIKE O.CDNVLARVPROD||'%') AND (M.CDDESPPREV IS NULL))
                                         OR ((M.CDDESPPREV = O.CDDESPPREV) AND (M.CDDESPPREV IS NOT NULL)))
                 GROUP BY O.IDORIGEM, O.IDTPDEMANDA, O.CDNVLARVPROD"
        );
		$stmt->execute($filters);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("getTabelaOrcaConsumo", $result));
    }
    
    public function getResumoDemonstrativoGrid(Request $request, Response $response) {
        $cdfilial = $this->environment->getCdFilial();

        try{
            $nrlista = $this->insertLista($cdfilial);
            $filters = array(
                "P_CDFILIAL" => $cdfilial,
                "P_NRLISTA" => $nrlista,
                "NRORG" => $this->environment->getNrorg()
            );
            $stmt = $this->connection->prepare(
                "SELECT T.DSDESCRICAOTOP AS NMITEM,
                        T.VRREAL,
                        T.VRPERREAL AS PERCENTREAL,
                        T.VRORCADO AS VRORCADO,
                        T.VRPERORCADO AS PERCENTORCADO,
                        (T.VRREAL - T.VRPERORCADO) AS DIFERENCA
                   FROM PARFILIALROP P, PARAMFILIAL PF, PARAMGERAL PG, TABLE(RESUMO_ROP(:P_NRLISTA, FIRST_DAY(TRUNC(CASE WHEN NVL(PG.IDUTILDTPROC,'N') = 'N' THEN SYSDATE ELSE NVL(PF.DTPROCESSA, SYSDATE) END)), TRUNC(CASE WHEN NVL(PG.IDUTILDTPROC,'N') = 'N' THEN SYSDATE ELSE NVL(PF.DTPROCESSA, SYSDATE) END), 'N', 'GU', 'P', P.IDCALCPERCENTFAT)) T
                  WHERE P.CDFILIAL = TIPOPARAM_DRO(:P_CDFILIAL)
                    AND PF.CDFILIAL = :P_CDFILIAL
                    AND PG.CDPARGERAL = '0001'
                    AND P.NRORG = :NRORG
                    AND P.NRORG = T.NRORG
                    AND P.NRORG = PF.NRORG
                    AND P.NRORG = PG.NRORG"
            );
            $stmt->execute($filters);
            $result = $stmt->fetchAll();
            
            $this->deleteLista($nrlista);
            
            $response->addDataSet(new DataSet("getResumoDemonstrativoGrid", $result));
        
        } catch(CustomException $e) {
            if($nrlista){
                $this->deleteLista($nrlista);
            }
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
    
    //insere lista no banco e retorna nrlista 
    public function insertLista($cdlista) {
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $nrlista;
    }
    
    //apaga lista do banco
    public function deleteLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        
        $sql = "DELETE FROM LISTA WHERE NRLISTA =:NRLISTA";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function sendMailNotification(Request\Row $request, Response $response) 
    {
        $emailController = new Util\EmailController;
        
        $this->crudService     = $crudService;
        
        $row = $request->getRow();
        
        if (sizeof($row['NMFORNECED']) > 0) {
            $attrImg = 'width="200px"';
            $srcLogo = $this->emailController->getLogo();
            $lgpd = $this->emailController->getLgpd($row['CDFILIAL']);
            $this->emailController->getLogoAttributes($srcLogo, $attrImg);
            
            $subject = $row['ASSUNTOEMAIL'];
            $assunto = "<p class='bold' style='text-align: center; color: #ED2424'>" . $subject . "</p>";    
        
            $formattedList = array();
            
            if (strpos($row['TEXTOEMAIL'], '<LISTA AFS>')) {
                $list = $this->getDataToCreateList($row)->getRows();
                $formattedList = $this->formatList($list);
            }
        
            foreach ($row['NMFORNECED'] as $supplier) {
                $mensagem = $this->replaceTagsContent($supplier, $row, $formattedList);

                $emails = $this->emailController->getEmailsRelacionadosFornecedor($supplier['CDFORNECED']);
                
                if($emails) {
                    $template = $this->emailController->replaceMessageContent($srcLogo, $assunto, $mensagem, $attrImg, $lgpd);
                    $this->emailController->enviaEmail($subject, $template, $emails);
                }
            }
        }
    }
    
    private function getDataToCreateList($row) 
    {
        $conditions = [
            array(
                "columnName" => "P_CDFILCENTCMP",
                "operator"   => '=',
                "value"      => $row['CDFILCENTCMP']
            ),
            array(
                "columnName" => "P_INICIOMES",
                "operator"   => '=',
                "value"      => $row['DATA']
            ),
            array(
                "columnName" => "P_FIMMES",
                "operator"   => '=',
                "value"      => $row['DATA']
            )];

        return $this->crudService->getFilteredDataSet('getAFsWithoutNF', $conditions);
    }
    
    private function formatList($list) 
    {
        $formattedList = array();
        
        if (sizeof($list) > 0) {
            foreach ($list as $af) {
                $formattedList[$af['CDFORNECED']][$af['CDFILDEST']][$af['NRAUTOFORN']] = $af;
            }
        }
        
        return $formattedList;
    }
    
    private function replaceTagsContent($supplier, $row, $list) 
    {
        $textoEmail = nl2br($row['TEXTOEMAIL']);
        
        if (strpos($textoEmail, '<FORNECEDOR>')) {
            $textoEmail = str_replace('<FORNECEDOR>', $supplier['NMRAZSOCFORN'], $textoEmail);
        }
        
        if (sizeof($list) > 0) {
            $htmlList = $this->createSupplierAFList($supplier, $row, $list);
            $textoEmail = str_replace('<LISTA AFS>', $htmlList, $textoEmail);
        }
        
        return $textoEmail;
    }
    
    private function createSupplierAFList($supplier, $row, $list) 
    {
        $html = '';
        
        if (sizeof($list[$supplier['CDFORNECED']]) > 0) {
            $html .= '<p>Data: ' . $row['DATA'] . '</p>
                      <ul>';
            
            foreach ($list[$supplier['CDFORNECED']] as $supplierList) {
                $html .= '<li>Unidade: ' . $supplier['NMFILIAL'] . '</li>
                          <ul>';

                foreach ($supplierList as $af) {
                    $html .= '<li>AF: ' . $af['NRAUTOFORN'] . '</li>';
                }
                
                $html .= '</ul>';
            }
            
            $html .= '</ul>';
        }
        
        return $html;
    }
    
    public function getGaugeChartTrackingData(Request\Row $request, Response $response)
    {
        $row = $request->getRow();
        
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_NRINDICADORTF" => $row['P_NRINDICADORTF']
            );
            
            $stmt = $this->connection->prepare("
                 SELECT
                     DSINDICADOR,
                     TO_CHAR(SYSDATE - 1, 'DD/MM/YYYY') AS DATAAPURADO,
                     NVL(VRLIMMINIMO, 0) AS VRLIMMINIMO,
                     NVL(VRLIMMETA, 0) AS VRLIMMETA,
                     NVL(ROUND( ( ( (ENTREGA_TOTAL * 100) / NVL(DECODE(TOTAL_AF,0,1,TOTAL_AF),1) ) ),2), 0) AS ENTREGA_TOTAL
                 FROM
                     (
                         SELECT
                             ( SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE) ) AS TOTAL_AF,
                             SUM(DEVOLUCAO) AS DEVOLUCAO,
                             SUM(TOTAL) AS ENTREGA_TOTAL,
                             SUM(NAOENTREGUE) AS NAO_ENTREGUE,
                             SUM(PARCIAL) AS ENTREGA_PARCIAL,
                             SUM(PENDENTE) AS PENDENTE
                         FROM
                             (
                                SELECT * 
                                  FROM DM_TRACKING_AF
                                 WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                                   AND DTPROGENTAF = TRUNC(SYSDATE - 1)
                             ) PIVOT (
                                 COUNT ( NRAUTOFORN )
                                 FOR IDCONFRECEBIMENTO
                                 IN ( 'D' DEVOLUCAO,'T' TOTAL,'N' NAOENTREGUE,'P' PARCIAL,'X' PENDENTE )
                             )
                     ) DADOS,
                     INDICADORTF
                 WHERE
                     NRINDICADORTF =:P_NRINDICADORTF
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            
            $response->addDataSet(new DataSet("getGaugeChartTrackingData", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_CHART_550', 550));
        }
    }
    
    public function getLineChartTrackingData(Request\Row $request, Response $response)
    {
        $row = $request->getRow();
        
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP']
            );
            
            $stmt = $this->connection->prepare("
                SELECT
                    TO_CHAR(DTPROGENTAF,'DD/MM/YYYY') AS DTPROGENTAF,
                    ROUND( ( ( (ENTREGA_TOTAL * 100) / NVL(DECODE(TOTAL_AF,0,1,TOTAL_AF),1) ) ),2) AS ENTREGA_TOTAL,
                    ROUND( ( ( (DEVOLUCAO * 100) / NVL(DECODE(TOTAL_AF,0,1,TOTAL_AF),1) ) ),2) AS DEVOLUCAO,
                    ROUND( ( ( (NAO_ENTREGUE * 100) / NVL(DECODE(TOTAL_AF,0,1,TOTAL_AF),1) ) ),2) AS NAO_ENTREGUE,
                    ROUND( ( ( (ENTREGA_PARCIAL * 100) / NVL(DECODE(TOTAL_AF,0,1,TOTAL_AF),1) ) ),2) AS ENTREGA_PARCIAL,
                    ROUND( ( ( (PENDENTE * 100) / NVL(DECODE(TOTAL_AF,0,1,TOTAL_AF),1) ) ),2) AS PENDENTE
                FROM
                (
                    SELECT
                        DTPROGENTAF,
                        NRORG,
                        ( SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE) ) AS TOTAL_AF,
                        SUM(DEVOLUCAO) AS DEVOLUCAO,
                        SUM(TOTAL) AS ENTREGA_TOTAL,
                        SUM(NAOENTREGUE) AS NAO_ENTREGUE,
                        SUM(PARCIAL) AS ENTREGA_PARCIAL,
                        SUM(PENDENTE) AS PENDENTE
                    FROM
                    (
                        SELECT * 
                          FROM DM_TRACKING_AF
                         WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                           AND DTPROGENTAF BETWEEN  TRUNC(SYSDATE - 7) AND TRUNC(SYSDATE - 1)
                    ) 
                    PIVOT 
                    (
                        COUNT ( NRAUTOFORN )
                        FOR IDCONFRECEBIMENTO
                        IN ( 'D' DEVOLUCAO,'T' TOTAL,'N' NAOENTREGUE,'P' PARCIAL,'X' PENDENTE )
                    )
                    GROUP BY
                        DTPROGENTAF,
                        NRORG
                    ORDER BY
                        DTPROGENTAF
                )
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            
            $response->addDataSet(new DataSet("getLineChartTrackingData", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_CHART_551', 551));
        }
    }
    
    public function getPieChartTrackingData(Request\Row $request, Response $response)
    {
        $row = $request->getRow();
        
        switch ($row['P_WIDGETNAME']) {
            case 'ENTTOTAFMESANTGERAL':
                $this->returnPieGeneralInitialData($row, $response);
            break;
            
            case 'ENTTOTAFMESANTCOMP':
                $this->returnPieBuyerInitialData($row, $response);
            break;
            
            case 'AFSCONFFORN':
                $this->returnPieAFConfInitialData($row, $response);
            break;
            
            default:
                $response->setError(new Error('ERR_FIND_WIDGET_552', 552));
            break;
        }
    }
    
    private function returnPieGeneralInitialData($row, $response)
    {
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
            );
            
            $stmt = $this->connection->prepare("
                SELECT 
                    CASE
                        IDCONFRECEBIMENTO
                        WHEN 'T' THEN 'Entrega Total'
                        WHEN 'P' THEN 'Entrega Parcial'
                        WHEN 'N' THEN 'Não Entregue'
                        WHEN 'D' THEN 'Devolução'
                        ELSE 'Pendente'
                    END AS LABEL,
                    COUNT(CDFILDEST) AS VALUE,
                    NVL(IDCONFRECEBIMENTO, 'X') AS IDCONFRECEBIMENTO
                FROM
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                )
                GROUP BY
                    IDCONFRECEBIMENTO
                ORDER BY
                    DECODE(IDCONFRECEBIMENTO, 'T', 1, 'P', 2, 'N', 3, 'D', 4, 5)
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            
            $response->addDataSet(new DataSet("getPieChartTrackingData", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_CHART_553', 553));
        }
    }
    
    private function returnPieBuyerInitialData($row, $response)
    {
        try {
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
            );
            
            $stmt = $this->connection->prepare("
                SELECT 
                    NMOPERADOR AS LABEL,
                    COUNT(NRAUTOFORN) AS VALUE, 
                    CDOPERADOR
                FROM 
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                )
                GROUP BY 
                    CDOPERADOR,
                    NMOPERADOR
                ORDER BY 
                    CDOPERADOR,
                    NMOPERADOR
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            
            $response->addDataSet(new DataSet("getPieChartTrackingData", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_CHART_554', 554));
        }
    }
    
    private function returnPieAFConfInitialData($row, $response)
    {
        try {
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
            );
            
            $stmt = $this->connection->prepare("
                SELECT 
                    CASE
                        IDSITUATENDIMENTOAF
                        WHEN 'C' THEN 'Confirmada'
                        WHEN 'P' THEN 'Parcial'
                        WHEN 'R' THEN 'Recusada'
                        ELSE 'Pendente'
                    END as LABEL,
                    COUNT(NRAUTOFORN) AS VALUE, 
                    NVL(IDSITUATENDIMENTOAF,'X') AS IDCONFRECEBIMENTO
                FROM 
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(LAST_DAY(SYSDATE))
                )
                GROUP BY
                    IDSITUATENDIMENTOAF
                ORDER BY
                    DECODE(NVL(IDSITUATENDIMENTOAF, 'X'), 'C', 1, 'X', 2, 'P', 3, 4)
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            
            $response->addDataSet(new DataSet("getPieChartTrackingData", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_CHAT_555', 555));
        }
    }
    
    public function getStackedChartTrackingData(Request\Row $request, Response $response){
        $row = $request->getRow();
        
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_NRINDICADORTF" => $row['P_NRINDICADORTF']
            );
            
            $stmt = $this->connection->prepare("
                SELECT
                    DTPROGENTAF AS LABEL,
                    (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE)) AS TOTAL_AF,
                    ROUND(((SUM(TOTAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS TOTAL,
                    ROUND(((SUM(PARCIAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PARCIAL,
                    ROUND(((SUM(NAOENTREGUE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS NAOENTREGUE,
                    ROUND(((SUM(DEVOLUCAO) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS DEVOLUCAO,
                    ROUND(((SUM(PENDENTE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PENDENTE,
                    VRLIMMETA
                FROM (
                    SELECT
                        CDFILDEST,
                        NMFILIAL,
                        NRAUTOFORN,
                        CDFORNECED,
                        NVL(IDCONFRECEBIMENTO,'X') AS IDCONFRECEBIMENTO,
                        TO_CHAR(DTPROGENTAF, 'MM/YYYY') AS DTPROGENTAF
                    FROM (
                        SELECT * 
                          FROM DM_TRACKING_AF
                         WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                           AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(ADD_MONTHS(SYSDATE, -12))) AND TRUNC(LAST_DAY(ADD_MONTHS(SYSDATE, -1)))
                        )
                    )
                ) 
                PIVOT (
                    COUNT ( NRAUTOFORN )
                    FOR IDCONFRECEBIMENTO
                    IN ( 'D' DEVOLUCAO,'T' TOTAL,'N' NAOENTREGUE,'P' PARCIAL,'X' PENDENTE )
                ) DADOS,
                INDICADORTF
                WHERE NRINDICADORTF = :P_NRINDICADORTF
                GROUP BY
                    DTPROGENTAF,
                    VRLIMMETA
                ORDER BY
                    TO_DATE(DTPROGENTAF, 'MM/YYYY')
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            
            $response->addDataSet(new DataSet("getStackedChartTrackingData", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_CHART_556', 556));
        }
    }
    
    public function drillDownPieChartTracking(Request\Row $request, Response $response){
        $row = $request->getRow();
        
        switch ($row['P_WIDGETNAME']) {
            case 'ENTTOTAFMESANTGERAL':
            case 'ENTTOTAFMESANTCOMP':
                $this->returnEntTotAFDrillDownData($row, $response);
            break;
            
            case 'AFSCONFFORN':
                $this->returnAFsConfFornDrillDownData($row, $response);
            break;
            
            default:
                $response->setError(new Error('ERR_FIND_WIDGET_557', 557));
            break;
        }
    }
     
    private function returnEntTotAFDrillDownData($row, $response){
        switch ($row['P_LEVEL']) {
            case '1':
                if ($row['P_WIDGETNAME'] == 'ENTTOTAFMESANTGERAL') {
                    $this->returnPieGeneralEntTotAFDataFirstLevel($row, $response);    
                } else {
                    $this->returnPieBuyerEntTotAFDataFirstLevel($row, $response);    
                }
            break;
            
            case '2':
                $this->returnPieEntTotAFDataSecondLevel($row, $response);  
            break;
            
            case '3':
                $this->returnPieEntTotAFDataThirdLevel($row, $response);  
            break;
            
            default:
                $response->setError(new Error('ERR_LEVEL_DET_558', 558));
            break;
        }
    }
    
    private function returnPieGeneralEntTotAFDataFirstLevel($row, $response){
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_IDCONFRECEBIMENTO" => $row['P_IDCONFRECEBIMENTO']
            );
            
            $stmt = $this->connection->prepare("
                SELECT 
                    NMOPERADOR AS LABEL,
                    COUNT(NRAUTOFORN) AS VALUE,
                    NVL(IDCONFRECEBIMENTO, 'X') AS IDCONFRECEBIMENTO, 
                    CDOPERADOR
                FROM 
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                       AND NVL(IDCONFRECEBIMENTO, 'X') = :P_IDCONFRECEBIMENTO
                )
                GROUP BY 
                    IDCONFRECEBIMENTO,
                    CDOPERADOR,
                    NMOPERADOR
                ORDER BY 
                    CDOPERADOR
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();

            $response->addDataSet(new DataSet("drillDownPieChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_559', 559));
        }
    }
    
    private function returnPieBuyerEntTotAFDataFirstLevel($row, $response){
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_CDOPERADOR" => $row['P_OPERADOR']
            );
            
            $stmt = $this->connection->prepare("
                SELECT 
                    CASE
                        IDCONFRECEBIMENTO
                        WHEN 'T' THEN 'Total'
                        WHEN 'P' THEN 'Parcial'
                        WHEN 'N' THEN 'Nâo Entregue'
                        WHEN 'D' THEN 'Devolucao'
                        ELSE 'Pendente'
                    END AS LABEL,
                    COUNT(CDFILDEST) AS VALUE,
                    CDOPERADOR,
                    NVL(IDCONFRECEBIMENTO, 'X') AS IDCONFRECEBIMENTO
                FROM 
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                       AND CDOPERADOR = :P_CDOPERADOR
                )
                GROUP BY 
                    IDCONFRECEBIMENTO,
                    CDOPERADOR
                ORDER BY
                    DECODE(IDCONFRECEBIMENTO, 'T', 1, 'P', 2, 'N', 3, 'D', 4, 5)
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();

            $response->addDataSet(new DataSet("drillDownPieChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_560', 560));
        }
    }
    
    private function returnPieEntTotAFDataSecondLevel($row, $response){
        try{
            $params = array(
                "P_CDOPERADOR" => $row['P_OPERADOR'],
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_IDCONFRECEBIMENTO" => $row['P_IDCONFRECEBIMENTO']
            );
            $stmt = $this->connection->prepare("
                SELECT
                    NMREGIAOGEO AS LABEL,
                    COUNT(NRAUTOFORN) AS VALUE,
                    NVL(IDCONFRECEBIMENTO,'X') AS IDCONFRECEBIMENTO,
                    CDOPERADOR,
                    NMREGIAOGEO AS REGIAO
                FROM
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                       AND CDOPERADOR = :P_CDOPERADOR
                       AND NVL(IDCONFRECEBIMENTO, 'X') = :P_IDCONFRECEBIMENTO
                )
                GROUP BY
                    NMREGIAOGEO,
                    CDOPERADOR,
                    IDCONFRECEBIMENTO
                ORDER BY
                    NMREGIAOGEO
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            
            $response->addDataSet(new DataSet("drillDownPieChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_561', 561));
        }
    }
    
    private function returnPieEntTotAFDataThirdLevel($row, $response){
        try{
            $params = array(
                "P_NMREGIAOGEO" => $row['P_NMREGIAOGEO'],
                "P_CDOPERADOR" => $row['P_OPERADOR'],
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_IDCONFRECEBIMENTO" => $row['P_IDCONFRECEBIMENTO']
            );
            
            $stmt = $this->connection->prepare("
                SELECT
                    NMESTADO AS LABEL,
                    COUNT(NRAUTOFORN) AS VALUE,
                    NVL(IDCONFRECEBIMENTO,'X') AS IDCONFRECEBIMENTO,
                    CDOPERADOR,
                    NMREGIAOGEO AS REGIAO,
                    NMESTADO
                FROM
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                       AND CDOPERADOR = :P_CDOPERADOR
                       AND NVL(IDCONFRECEBIMENTO,'X') = :P_IDCONFRECEBIMENTO
                       AND NMREGIAOGEO = :P_NMREGIAOGEO
                )
                GROUP BY
                    NMESTADO,
                    NMREGIAOGEO,
                    CDOPERADOR,
                    IDCONFRECEBIMENTO
                ORDER BY
                    NMESTADO
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            
            $response->addDataSet(new DataSet("drillDownPieChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_562', 562));
        }
    }
    
    private function returnAFsConfFornDrillDownData($row, $response){
        switch ($row['P_LEVEL']) {
            case '1':
                $this->returnPieAFsConfFornDataFirstLevel($row, $response);    
            break;
            
            case '2':
                $this->returnPieAFsConfFornDataSecondLevel($row, $response);  
            break;
            
            case '3':
                $this->returnPieAFsConfFornDataThirdLevel($row, $response);  
            break;
            
            default:
                $response->setError(new Error('ERR_LEVEL_DET_563', 563));
            break;
        }
    }
    
    private function returnPieAFsConfFornDataFirstLevel($row, $response){
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_IDSITUATENDIMENTOAF" => $row['P_IDCONFRECEBIMENTO']
            );
            
            $stmt = $this->connection->prepare("
                SELECT 
                    NMOPERADOR AS LABEL,
                    COUNT(NRAUTOFORN) AS VALUE, 
                    NVL(IDSITUATENDIMENTOAF,'X') AS IDCONFRECEBIMENTO,
                    CDOPERADOR
                FROM 
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(LAST_DAY(SYSDATE))
                       AND NVL(IDSITUATENDIMENTOAF, 'X') = :P_IDSITUATENDIMENTOAF
                )
                GROUP BY 
                    IDSITUATENDIMENTOAF,
                    CDOPERADOR,
                    NMOPERADOR
                ORDER BY
                    CDOPERADOR
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();

            $response->addDataSet(new DataSet("drillDownPieChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_564', 564));
        }
    }
    
    private function returnPieAFsConfFornDataSecondLevel($row, $response){
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_IDSITUATENDIMENTOAF" => $row['P_IDCONFRECEBIMENTO'],
                "P_CDOPERADOR" => $row['P_OPERADOR']
            );
            
            $stmt = $this->connection->prepare("
                SELECT 
                    NMREGIAOGEO AS LABEL,
                    COUNT(NRAUTOFORN) AS VALUE,
                    NVL(IDSITUATENDIMENTOAF,'X') AS IDCONFRECEBIMENTO,
                    CDOPERADOR, 
                    NMREGIAOGEO AS REGIAO
                FROM 
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(LAST_DAY(SYSDATE))
                       AND NVL(IDSITUATENDIMENTOAF, 'X') = :P_IDSITUATENDIMENTOAF
                       AND CDOPERADOR = :P_CDOPERADOR
                )
                GROUP BY 
                    NMREGIAOGEO,
                    IDSITUATENDIMENTOAF,
                    CDOPERADOR
                ORDER BY
                    NMREGIAOGEO
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();

            $response->addDataSet(new DataSet("drillDownPieChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_565', 565));
        }
    }
    
    private function returnPieAFsConfFornDataThirdLevel($row, $response){
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_IDSITUATENDIMENTOAF" => $row['P_IDCONFRECEBIMENTO'],
                "P_CDOPERADOR" => $row['P_OPERADOR'],
                "P_NMREGIAOGEO" => $row['P_NMREGIAOGEO']
            );
            
            $stmt = $this->connection->prepare("
                SELECT 
                    NMESTADO AS LABEL,
                    COUNT(NRAUTOFORN) AS VALUE, 
                    NVL(IDSITUATENDIMENTOAF,'X') AS IDCONFRECEBIMENTO,
                    CDOPERADOR, 
                    NMREGIAOGEO AS REGIAO, 
                    NMESTADO
                FROM 
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(LAST_DAY(SYSDATE))
                       AND NVL(IDSITUATENDIMENTOAF, 'X') = :P_IDSITUATENDIMENTOAF
                       AND CDOPERADOR = :P_CDOPERADOR
                       AND NMREGIAOGEO = :P_NMREGIAOGEO
                )
                GROUP BY 
                    NMESTADO,
                    NMREGIAOGEO,
                    IDSITUATENDIMENTOAF,
                    CDOPERADOR
                ORDER BY
                    NMESTADO
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();

            $response->addDataSet(new DataSet("drillDownPieChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_566', 566));
        }
    }
    
    public function drillDownStackedChartTracking(Request\Row $request, Response $response){
        $row = $request->getRow();
        
        switch ($row['P_LEVEL']) {
            case '1':
                $this->returnStackedDataFirstLevel($row, $response);    
            break;
            
            case '2':
                $this->returnStackedDataSecondLevel($row, $response);  
            break;
            
            case '3':
                $this->returnStackedDataThirdLevel($row, $response);  
            break;
            
            default:
                $response->setError(new Error('ERR_LEVEL_DET_567', 567));
            break;
        }
    }
    
    private function returnStackedDataFirstLevel($row, $response){
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_STARTDATE" => $row['P_STARTDATE'],
                "P_ENDDATE" => $row['P_ENDDATE']
            );
            
            $stmt = $this->connection->prepare("
                SELECT
                    NMOPERADOR AS LABEL,
                    (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE)) AS TOTAL_AF,
                    ROUND(((SUM(TOTAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS TOTAL,
                    ROUND(((SUM(PARCIAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PARCIAL,
                    ROUND(((SUM(NAOENTREGUE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS NAOENTREGUE,
                    ROUND(((SUM(DEVOLUCAO) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS DEVOLUCAO,
                    ROUND(((SUM(PENDENTE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PENDENTE,
                    :P_STARTDATE AS STARTDATE,
                    :P_ENDDATE AS ENDDATE,
                    CDOPERADOR
                FROM
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(TO_DATE(:P_STARTDATE, 'DD/MM/YYYY')) AND TRUNC(TO_DATE(:P_ENDDATE, 'DD/MM/YYYY'))
                )
                PIVOT 
                (
                    COUNT ( NRAUTOFORN )
                    FOR IDCONFRECEBIMENTO
                    IN ( 'D' DEVOLUCAO,'T' TOTAL,'N' NAOENTREGUE,'P' PARCIAL,'X' PENDENTE )
                )
                GROUP BY
                    NMOPERADOR,
                    CDOPERADOR
                ORDER BY
                    CDOPERADOR
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();

            $response->addDataSet(new DataSet("drillDownStackedChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_568', 568));
        }
    }
    
    private function returnStackedDataSecondLevel($row, $response){
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_STARTDATE" => $row['P_STARTDATE'],
                "P_ENDDATE" => $row['P_ENDDATE'],
                "P_CDOPERADOR" => $row['P_OPERADOR']
            );
            $stmt = $this->connection->prepare("
                SELECT
                    NMREGIAOGEO AS LABEL, 
                    (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE)) AS TOTAL_AF,
                    ROUND(((SUM(TOTAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS TOTAL,
                    ROUND(((SUM(PARCIAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PARCIAL,
                    ROUND(((SUM(NAOENTREGUE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS NAOENTREGUE,
                    ROUND(((SUM(DEVOLUCAO) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS DEVOLUCAO,
                    ROUND(((SUM(PENDENTE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PENDENTE,
                    :P_STARTDATE AS STARTDATE,
                    :P_ENDDATE AS ENDDATE,
                    CDOPERADOR,
                    NMREGIAOGEO
                FROM 
                (
                    SELECT  NMOPERADOR,
                            NRAUTOFORN,
                            NVL(IDCONFRECEBIMENTO, 'X') AS IDCONFRECEBIMENTO, 
                            CDOPERADOR,
                            NMESTADO,
                            NMREGIAOGEO
                       FROM DM_TRACKING_AF
                      WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                        AND DTPROGENTAF BETWEEN TRUNC(TO_DATE(:P_STARTDATE, 'DD/MM/YYYY')) AND TRUNC(TO_DATE(:P_ENDDATE, 'DD/MM/YYYY'))
                        AND CDOPERADOR = :P_CDOPERADOR
                )
                PIVOT 
                (
                    COUNT(NRAUTOFORN)
                    FOR IDCONFRECEBIMENTO IN ('D' DEVOLUCAO, 'T' TOTAL, 'N' NAOENTREGUE, 'P' PARCIAL, 'X' PENDENTE)
                )
                GROUP BY 
                    NMREGIAOGEO,
                    CDOPERADOR
                ORDER BY 
                    NMREGIAOGEO
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();

            $response->addDataSet(new DataSet("drillDownStackedChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_569', 569));
        }
    }
    
    private function returnStackedDataThirdLevel($row, $response){
        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_STARTDATE" => $row['P_STARTDATE'],
                "P_ENDDATE" => $row['P_ENDDATE'],
                "P_CDOPERADOR" => $row['P_OPERADOR'],
                "P_NMREGIAOGEO" => $row['P_NMREGIAOGEO']
            );
            
            $stmt = $this->connection->prepare("
                SELECT 
                    NMESTADO AS LABEL,
                    TOTAL_AF,
                    ROUND((((ENTREGA_TOTAL * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as TOTAL,
                    ROUND((((ENTREGA_PARCIAL * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as PARCIAL,
                    ROUND((((NAO_ENTREGUE * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as NAOENTREGUE,
                    ROUND((((DEVOLUCAO * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as DEVOLUCAO,
                    ROUND((((PENDENTE * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as PENDENTE,
                    :P_STARTDATE AS STARTDATE,
                    :P_ENDDATE AS ENDDATE,
                    CDOPERADOR,
                    NMREGIAOGEO,
                    NMESTADO
                FROM 
                (
                    SELECT 
                        NMESTADO, 
                        CDOPERADOR,
                        NMREGIAOGEO,
                        (SUM(DEVOLUCAO)+SUM(TOTAL)+ SUM(NAOENTREGUE)+ SUM(PARCIAL) + SUM(PENDENTE)) AS TOTAL_AF,
                        SUM(DEVOLUCAO) AS DEVOLUCAO, 
                        SUM(TOTAL) AS ENTREGA_TOTAL, 
                        SUM(NAOENTREGUE) AS NAO_ENTREGUE, 
                        SUM(PARCIAL) AS ENTREGA_PARCIAL, 
                        SUM(PENDENTE) AS PENDENTE
                    FROM 
                    (
                        SELECT NMOPERADOR,
                               NRAUTOFORN, 
                               NVL(IDCONFRECEBIMENTO, 'X') AS IDCONFRECEBIMENTO, 
                               CDOPERADOR, 
                               NMESTADO,
                               NMREGIAOGEO
                          FROM DM_TRACKING_AF
                         WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                           AND DTPROGENTAF BETWEEN TRUNC(TO_DATE(:P_STARTDATE, 'DD/MM/YYYY')) AND TRUNC(TO_DATE(:P_ENDDATE, 'DD/MM/YYYY'))
                           AND CDOPERADOR = :P_CDOPERADOR
                           AND NMREGIAOGEO = :P_NMREGIAOGEO
                    )
                    PIVOT 
                    (
                        COUNT(NRAUTOFORN)
                        FOR IDCONFRECEBIMENTO IN ('D' DEVOLUCAO, 'T' TOTAL, 'N' NAOENTREGUE, 'P' PARCIAL, 'X' PENDENTE)
                    )
                    GROUP BY 
                        NMESTADO,
                        CDOPERADOR,
                        NMREGIAOGEO
                    ORDER BY 
                        NMESTADO
                )
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();

            $response->addDataSet(new DataSet("drillDownStackedChartTracking", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_DRILLDOWN_570', 570));
        }
    }
    
    public function getGeoChartData(Request\Row $request, Response $response){
        $row = $request->getRow();

        try{
            $params = array(
                "P_CDFILCENTCMP" => $row['P_CDFILCENTCMP'],
                "P_STARTDATE" => $row["P_STARTDATE"],
                "P_ENDDATE" => $row["P_ENDDATE"]
            );
            
            $stmt = $this->connection->prepare("
                SELECT
                    SGESTADO,
                    ABS(ROUND(AVG(( DTENTRSAID - DTCONFRECEBIMENTO )),1) ) AS TEMPO_MEDIO
                FROM
                (
                    SELECT *
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :P_CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(TO_DATE(:P_STARTDATE, 'DD/MM/YYYY')) AND TRUNC(TO_DATE(:P_ENDDATE, 'DD/MM/YYYY'))
                )
                GROUP BY
                    SGESTADO
                ORDER BY
                    SGESTADO
            ");
            
            $stmt->execute($params);
            $result = $stmt->fetchAll();

            $response->addDataSet(new DataSet("geoChartData", $result));
        } catch(\Exception $e) {
            $response->setError(new Error('ERR_LOADING_MAP_571', 571));
        }
    }
} 